
//
//	Source code for the CAec Class library
//	7/7/97
//	Adrienne Electronics Corporation
//
#include <windows.h>
#include "CAecBrd.h"	//also has "   #include "Aec_nttc.h"   "
// ********************************************************************
//CAecBrd.cpp
CAec::CAec()	//constructor
{
	ISRRun = FALSE;
	ComTimeout = 50;	// 50ms timeout default, Public (alterable)
	IntTimeout = 10;	// 10ms timeout if command sent under ISR conditions
	exiting = FALSE;
	SonySetup = FALSE;
	FoundCnt = 0;		// changed when board is found
	RegCnt = 0;			// value returned from DLL
	IntPending = FALSE;	// default value
	Offset0 = 0;
	Offset3F0 = 0x03F0;
	Offset3F2 = 0x03F2;
	Offset3FD = 0x03FD;
	Offset3FE = 0x03FE;
	Offset3FF = 0x03FF;
	selection = 0;
	Find();
}

// --------------------------------------------------------------------
BOOL CAec::check(int i)	//private function
{
	// if reading one byte returns TRUE, a board is registered
	// at this address! Exit if no board is registered
	OffsetR = 0x0000;	//base address of all boards
	OffsetW = 0x03FF;	// command input address
	booltemp2 = AEC_PCTC_READ_1BYTE(&OneBR,  &OffsetR, &Addr);
	//booltemp2 = AEC_PCTC_WRITE_1BYTE	(LPBYTE,  LPWORD, LPWORD);
	if (booltemp2)
	{
		CkType = AEC_PCTC_CHECK_BOARD (&Addr);		
		if	(CkType == 0 )  // if device is registered but does not check-out,
		{					// try sending the reset byte and try again
			OneBW = 0x02;
			booltemp2 = AEC_PCTC_WRITE_1BYTE(&OneBW, &OffsetW, &Addr);	
			Sleep(3000);	// allow time for board to to boot
		}
	}
	CkType = AEC_PCTC_CHECK_BOARD (&Addr);		
	if	(CkType != 0 )
	{
		AecBoard[i].Address = Addr;
		AecBoard[i].BrdCkType = CkType;
		booltemp2 = TRUE;
	}
	else booltemp2 = FALSE;
	return booltemp2;

}

// --------------------------------------------------------------------
BYTE CAec::Read1(WORD offset)
{
	if (CkType == 1)
	{
		if  (offset > 0x0009)
			offset = 0x0009;
		ok = AEC_PCTC_READ_1BYTE (&OneBR, &offset, &Addr);
		return OneBR;
	}

	if ((CkType == 2) && (offset > 0x0009))
		offset = 0x0009;
	else if ((CkType == 3) && (offset > 0x03FF))
		offset = 0x03FF;
	else if (CkType == 0)
		return 0x00;
	return 	*(lpBrd + offset);	//if smart or MMR, read byte and return
}
// --------------------------------------------------------------------
BOOL CAec::Write1(BYTE byte, WORD offset)
{
	if (CkType != 3)
		return FALSE;
	if (offset > 0x03FF)
		return FALSE;
	*(lpBrd + offset) = byte;
	return TRUE;
}
// --------------------------------------------------------------------
DWORD CAec::Read4()
{
	if (CkType == 1)
	{
		OffsetR = 0;
		ok = AEC_PCTC_READ_4BYTES (&FourBR, &OffsetR, &Addr);
		return FourBR;
	}
	return 	*((LPDWORD)lpBrd);
}

// --------------------------------------------------------------------
DWORD CAec::Read4(WORD offset)
{
	if (CkType == 1)
	{
		offset = 0;
		ok = AEC_PCTC_READ_4BYTES (&FourBR, &offset, &Addr);
		return FourBR;
	}
	if (CkType == 2)
		offset = 0x0000;
	if ((CkType == 3) && (offset > 0x03FC))
		offset = 0x03FC;
	return 	*((LPDWORD)(lpBrd + offset));
}
// --------------------------------------------------------------------
BOOL CAec::Write4(DWORD dwDat, WORD offset)
{
	if ((CkType != 3) || (offset > 0x03FC))
		return FALSE;
	*((LPDWORD)(lpBrd + offset)) = dwDat;
	return TRUE;
}
// --------------------------------------------------------------------
WORD CAec::BrdAddr()		//address used with DLL calls
{
	return Addr;
}
// --------------------------------------------------------------------
int	CAec::BrdType()		//1=IOR,2=MMR,3=SMART 
{
	return CkType;
}
// --------------------------------------------------------------------
BYTE CAec::BrdModel()		//offset 3EF (VLTC=50h,VITC=93h,LTC=98h)
{
	return Model;
}
// --------------------------------------------------------------------
BOOL CAec::ReadsLtc()
{
	return LRdr;		//True if board reads LTC
}
// --------------------------------------------------------------------
BOOL CAec::ReadsVitc()
{
	return VRdr;		//True if board reads VITC
}
// --------------------------------------------------------------------
BOOL CAec::GensLtc()
{
	return LGen;	//True if board gens LTC
}
// --------------------------------------------------------------------
BOOL CAec::GensVitc()
{
	return VGen;		//True if board gens VITC
}
// --------------------------------------------------------------------
BOOL CAec::HasSerial()
{
	return Serial;		//does time code board have serial option (no/yes)
}
// --------------------------------------------------------------------
BYTE CAec::ModelType()
{
	return ModType;	//'3'=RDR, '5'=RG1, '6'=RG2, '7'=RG3, '8'=GEN
}
// --------------------------------------------------------------------
BOOL CAec::Select(int i)	//public function
{
	
	if (i >= FoundCnt)	// do not try to select a board that is not here
		return FALSE;
	selection = i;
/*
	WORD	Addr;		//Value to be used in address field when calling DLL functions
	int		CkType;		//Value from "CheckBoard (DLL)1=IOR,2=MMR,3=SMART (0 not saved)
	BYTE	Model;		//Holds data from offset 3EF (VLTC=50h,VITC=93h,LTC=98h)
	BYTE	Id0;		//Data Byte from 3E0 First letter of ID
	BYTE	Id1;		//Data Byte from 3E1 Second letter of ID
	BYTE	Id2;		//Data Byte from 3E2 Third letter of ID
	BYTE	SLtr;		//Data Byte from 3E3 Board Software Revision letter
	BYTE	SNum;		//Data Byte from 3E4 Board Software Revision number
	BYTE	VerUp;		//Data Byte from 3E5 Upper byte of EPROM version
	BYTE	VerLow;		//Data Byte from 3E6 Lower byte of EPROM version
	BOOL	Serial;		//does time code board have serial option (no/yes)
	PVOID	PhysAddr;	//value of pointer to board
*/	

	Addr = AecBoard[i].Address;
	CkType = AecBoard[i].BrdCkType;

	if (CkType == 3)
	{
		Offset3Ex = 0x03E6;
		ok = AEC_PCTC_READ_1BYTE (&VerLow, &Offset3Ex, &Addr);
		Offset3Ex = 0x03E5;
		ok = AEC_PCTC_READ_1BYTE (&VerUp, &Offset3Ex, &Addr);	
		Offset3Ex = 0x03E4;
		ok = AEC_PCTC_READ_1BYTE (&SNum, &Offset3Ex, &Addr);	
		Offset3Ex = 0x03E3;
		ok = AEC_PCTC_READ_1BYTE (&SLtr, &Offset3Ex, &Addr);	
		// read the smart board type id byte ( 0x50 VLTC, 0x93 VITC, 0x98 LTC)
		Offset3Ex = 0x03EF;
		ok = AEC_PCTC_READ_1BYTE (&Model, &Offset3Ex, &Addr);
		// read the board id bytes to see if this board reads tc.
		Offset3Ex = 0x03E0;
		ok = AEC_PCTC_READ_1BYTE (&Id0, &Offset3Ex, &Addr);
		Offset3Ex = 0x03E1;
		ok = AEC_PCTC_READ_1BYTE (&Id1, &Offset3Ex, &Addr);	
		Offset3Ex = 0x03E2;
		ok = AEC_PCTC_READ_1BYTE (&Id2, &Offset3Ex, &Addr);	

		LRdr = FALSE;		//True if board reads LTC
		LGen = FALSE;		//True if board gens LTC
		VRdr = FALSE;		//True if board reads VITC
		VGen = FALSE;		//True if board gens VITC
		//ModType;			//'3'=RDR, '5'=RG1, '6'=RG2, '7'=RG3, '8'=GEN

		if ((Id0 == 'R' ) && (Id1 == 'D' ) && (Id2 == 'R' ))
			ModType = 3;
	
		else if ((Id0 == 'R' ) && (Id1 == 'G' ) && (Id2 == '1' ))
			ModType = 5;
	
		else if ((Id0 == 'R' ) && (Id1 == 'G' ) && (Id2 == '2' ))
			ModType = 6;

		else if ((Id0 == 'R' ) && (Id1 == 'G' ) && (Id2 == '3' ))
			ModType = 7;

		else if ((Id0 == 'G' ) && (Id1 == 'E' ) && (Id2 == 'N' ))
			ModType = 8;

		else if (Id0 == 'L')
		{
			if (Id2 == 'D')
				ModType = 10;
			else if (Id2 == 'G')
				ModType = 11;
			else //(Id2 == 'N')
				ModType = 12;
		}
		else if (Id0 == 'V')
		{
			if (Id2 == 'D')
				ModType = 13;
			else if (Id2 == 'G')
				ModType = 14;
			else //(Id2 == 'N')
				ModType = 15;
		}

		else	// unknown
			ModType = 0;

	// set the ability values (LRdr, LGen, VRdr, VGen)
		switch (Model) 
		{
			case 0x50:
				switch (ModType)
				{
					case 3: //RDR
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x23;	
						autogen = 0x00;	
						ltcjam = 0x00;		
						ltcread = 0x21;	
						vitcread = 0x22;	
						ltcgen = 0x00;		
						ltcrg = 0x00;		
						vtol = 0x00;		
						vitcgen = 0x00;	
						ltov = 0x00;		
						jamgenall = 0x00;	
						jamgentb = 0x00;	
						jamgenub = 0x00;	
						genstart = 0x00;	
						genstop = 0x00;	
						break;			

					case 5:  //RG1
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x23;	
						autogen = 0x2C;	
						ltcjam = 0x1A;		
						ltcread = 0x21;	 
						vitcread = 0x22;	 
						ltcgen = 0x24;		 
						ltcrg = 0x25;		 
						vtol = 0x26;		 
						vitcgen = 0x28;	 
						ltov = 0x29;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
			
					case 6:  //RG2 (RDR + LGEN)
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x23;	 
						autogen = 0x24;	 
						ltcjam = 0x1A;		 
						ltcread = 0x21;	 
						vitcread = 0x22;	 
						ltcgen = 0x24;		 
						ltcrg = 0x25;		 
						vtol = 0x26;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
			
					case 7:  //RG3 (RDR+VGEN)
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x23;	 
						autogen = 0x28;	 
						ltcjam = 0x1A;		 
						ltcread = 0x21;	 
						vitcread = 0x22;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x28;	 
						ltov = 0x29;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;

					case 8:  //GEN
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x00;	 
						autogen = 0x2C;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x00;	 
						ltcgen = 0x24;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x28;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
			
					case 10: //LRD
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x21;	 
						autogen = 0x00;	 
						ltcjam = 0x00;		 
						ltcread = 0x21;	 
						vitcread = 0x00;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x00;	 
						jamgentb = 0x00;	 
						jamgenub = 0x00;	 
						genstart = 0x00;	 
						genstop = 0x00;	 
						break;
			
					case 11: //LRG
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x21;	 
						autogen = 0x24;	 
						ltcjam = 0x1A;		 
						ltcread = 0x21;	 
						vitcread = 0x00;	 
						ltcgen = 0x24;		 
						ltcrg = 0x25;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
			
					case 12: //LGN
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x00;	 
						autogen = 0x24;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x00;	 
						ltcgen = 0x24;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;

					case 13: //VRD
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x22;	 
						autogen = 0x00;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x22;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x00;	 
						jamgentb = 0x00;	 
						jamgenub = 0x00;	 
						genstart = 0x00;	 
						genstop = 0x00;	 
						break;
	
					case 14: //VRG
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x22;	 
						autogen = 0x28;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x22;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x28;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;

					case 15: //VGN
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x00;	 
						autogen = 0x28;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x00;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x28;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
					//default: 
				}
				break;
			
			case 0x93: //VITC
				switch (ModType)
				{
					case 3:  //RDR
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x10;	 
						autogen = 0x00;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x10;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x00;	 
						jamgentb = 0x00;	 
						jamgenub = 0x00;	 
						genstart = 0x00;	 
						genstop = 0x00;	 
						break;

					case 5:  //RG1
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x10;	 
						autogen = 0x18;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x10;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x18;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;

					case 8:  //GEN
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x00;	 
						autogen = 0x18;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x00;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x18;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
					//default:
				}
				break;

			case 0x98: //LTC
				switch (ModType){
					case 3:  //RDR
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x10;	 
						autogen = 0x00;	 
						ltcjam = 0x00;		 
						ltcread = 0x10;	 
						vitcread = 0x00;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x00;	 
						jamgentb = 0x00;	 
						jamgenub = 0x00;	 
						genstart = 0x00;	 
						genstop = 0x00;	 
						break;

					case 5:  //RG1
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;
			
						//Model specific
						autoread = 0x10;	 
						autogen = 0x18;	 
						ltcjam = 0x1A;		 
						ltcread = 0x10;	 
						vitcread = 0x00;	 
						ltcgen = 0x18;		 
						ltcrg = 0x18;		  //*** Set other Bits ***
						vtol = 0x00;		 
						vitcgen = 0x00;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;

					case 8:  //GEN
						//ALL
						intack = 0x00;
						reset = 0x02;
						smpte = 0x03;
						ebu = 0x04;
						idle = 0x16;

						//Model specific
						autoread = 0x00;	 
						autogen = 0x18;	 
						ltcjam = 0x00;		 
						ltcread = 0x00;	 
						vitcread = 0x00;	 
						ltcgen = 0x00;		 
						ltcrg = 0x00;		 
						vtol = 0x00;		 
						vitcgen = 0x18;	 
						ltov = 0x00;		 
						jamgenall = 0x40;	 
						jamgentb = 0x41;	 
						jamgenub = 0x42;	 
						genstart = 0x48;	 
						genstop = 0x49;	 
						break;
					//default:
				}
				break;

			//default:	//no good options for default
		}

		Serial = CheckForSerial(); //BOOL yes/no Serial option
		PhysAddr = AEC_PCTC_GET_BOARD_POINTER (&Addr); 
				//value of pointer (PVOID) to board
		if ((LONG)PhysAddr != 0xFFFFFFFF)
		{
			lpBrd = (LPBYTE)PhysAddr;
			pointer = TRUE;
		}
		else
		{
			pointer = FALSE;
		}

		if (ltcread != 0x00)
			LRdr = TRUE;		//True if board reads LTC
		if (ltcgen != 0x00)
			LGen = TRUE;		//True if board gens LTC
		if (vitcread != 0x00)
			VRdr = TRUE;		//True if board reads VITC
		if (vitcgen != 0x00)
			VGen = TRUE;		//True if board gens VITC

		strcpy( strId, "PC-" );
		if (Model == 0x50)
			strcat( strId, "VLTC/" );
		else if (Model == 0x93)
			strcat( strId, "VITC/" );
		else if (Model == 0x98)
			strcat( strId, "LTC/" );
		strncat( strId, (char*)&Id0, 1 );
		strncat( strId, (char*)&Id1, 1 );
		strncat( strId, (char*)&Id2, 1 );
		strcat( strId, " Rev: " );

		strncat( strId, (char*)&SLtr, 1);
		strncat( strId, (char*)&SNum, 1);
		strncat( strId, (char*)&VerUp, 1);
		strncat( strId, (char*)&VerLow, 1);
		ok = TRUE;

	}
	// must be an IOR or MMR
	else if (CkType == 2) //MMR
	{
		strcpy( strId, "PC-LTC/MMR" );
		LRdr = TRUE;		//True if board reads LTC
		Serial = FALSE;
		PhysAddr = AEC_PCTC_GET_BOARD_POINTER (&Addr); 
			//value of pointer (PVOID) to board
		ok = TRUE;
	}
	else if (CkType == 1) //IOR
	{
		strcpy( strId, "PC-LTC/IOR" );
		LRdr = TRUE;		//True if board reads LTC
		Serial = FALSE;
		ok = TRUE;
	}

	else ok = FALSE;
	return (ok);
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Checks for 'Serial' option on Smart Boards by analyzing the return code
// from sending a 'serial' option specific command to a Smart time code
// board. The return code will be a '0xF0' if no serial option is present.
// If the serial option is present, the return code should be the same as
// the command.
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
BOOL	CAec::CheckForSerial()
{

	OneBW = 0x80; // Initialize the UART & data buffers
	ok = AEC_PCTC_SEND_COMMAND(&OneBW, &OneBR, &Addr);
	ok2 = (OneBW == OneBR);	// see if return value is same!
	if (OneBR == 0xF0)
		ok2 = FALSE;

	OneBW = 0xA0; // transmit the 'txdata' byte
	ok = AEC_PCTC_SEND_COMMAND(&OneBW, &OneBR, &Addr);
	ok2 = ((OneBW == OneBR) && ok2);	//
	if (OneBR == 0xF0)
		ok2 = FALSE;
	if (ok2)
	{
		inituart = 0x80;	// SERIAL
		initrx = 0x81;		// SERIAL
		readrx = 0x82;		// SERIAL
		txstat = 0x90;		// SERIAL 
		tx422en = 0x91;		// SERIAL
		tx422dis = 0x92;	// SERIAL
		txbreak = 0x93;		// SERIAL
		txbyte = 0xA0;		// SERIAL
		txstr1 = 0xA1;		//SERIAL
		txstr2 = 0xA2;		//SERIAL
		txstr3 = 0xA3;		//SERIAL
		txstr4 = 0xA4;		//SERIAL
		txstr5 = 0xA5;		//SERIAL
		txstr6 = 0xA6;		//SERIAL
		txstr7 = 0xA7;		//SERIAL
		txstr8 = 0xA8;		//SERIAL
		txstr9 = 0xA9;		//SERIAL
		txstrA = 0xAA;		//SERIAL
		txstrB = 0xAB;		//SERIAL
		txstrC = 0xAC;		//SERIAL
		txstrD = 0xAD;		//SERIAL
		txstrE = 0xAE;		//SERIAL
		txstrF = 0xAF;		//SERIAL
	}

	else
	{
		inituart = 0x00;	// SERIAL
		initrx = 0x00;		// SERIAL
		readrx = 0x00;		// SERIAL
		txstat = 0x00;		// SERIAL 
		tx422en = 0x00;		// SERIAL
		tx422dis = 0x00;	// SERIAL
		txbreak = 0x00;		// SERIAL
		txbyte = 0x00;		// SERIAL
		txstr1 = 0x00;		//SERIAL
		txstr2 = 0x00;		//SERIAL
		txstr3 = 0x00;		//SERIAL
		txstr4 = 0x00;		//SERIAL
		txstr5 = 0x00;		//SERIAL
		txstr6 = 0x00;		//SERIAL
		txstr7 = 0x00;		//SERIAL
		txstr8 = 0x00;		//SERIAL
		txstr9 = 0x00;		//SERIAL
		txstrA = 0x00;		//SERIAL
		txstrB = 0x00;		//SERIAL
		txstrC = 0x00;		//SERIAL
		txstrD = 0x00;		//SERIAL
		txstrE = 0x00;		//SERIAL
		txstrF = 0x00;		//SERIAL
	}

	return (ok2);

}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Call the 'Check_board' function for each of the possible addresses
//	that a time code board could be at and then store the board Type and
//	determine if the board has reading ability (some Smart boards are
//	generate only).  
// Returns the number of boards that where found.
//
//	PC-LTC/IOR BOARDS HAVE STANDARD ADDRESS JUMPER SETTINGS OF...
//	(port mapped)	200h, 220h, 280h, 2A0h, 300, 320h, 380h, & 3A0h
//	PC-LTC/MMR ... C0(00)h, C4(00)h, C8(00)h, CC(00)h, D0(00)h, D4(00)h
//	(memory mapped)		D8(00)h, & DC(00)h.
//	PC-LTC/(RDR, GEN, RG1) & ALL PC-VITC/XXX & ALL PC-VLTC/XXX HAVE
//		ADDRESS... CC00h, CD00h, CE00h, CF00h, DC00h, DD00h, DE00h, & DF00h
// The 'actual' address that the memory mapped boards are 'physically' 
//	mapped to is four bits higher than the jumper's lable. 
//	(ie: CC00 -> CC000h address
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//save the address and type of each board that is found in a structure
//default is to 'Select(0)' as default
void CAec::Find(void)
{

	FoundCnt = 0;
	RegCnt = AEC_PCTC_GET_NUMBER_OF_DEVICES ();	// returns number of boards registered.
	if (RegCnt == 0)
		return;	// no sense in going on if the DLL does not find any boards registered


//look for MMR/Smart return value at the possible memory map addresses

	Addr = 0xC000;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xC400;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xC800;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xCC00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xD000;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xD400;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xD800;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xDC00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xCD00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xCE00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xCF00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xDD00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xDE00;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0xDF00;
	if (check(FoundCnt))
		FoundCnt++;

//look for and IOR return value at the possible IOR addresses
	Addr = 0x200;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x220;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x280;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x2A0;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x300;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x320;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x380;
	if (check(FoundCnt))
		FoundCnt++;
	Addr = 0x3A0;
	if (check(FoundCnt))
		FoundCnt++;

	if (FoundCnt != 0)
		Select(selection);	// save the first board found as currently Selected
		
	return;
	
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
int CAec::GetCountFound()	//
{	
	  return FoundCnt;
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
int CAec::GetCountRegistered()	//
{	
	  return RegCnt;
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::AutoRead() //autoread
{

	if ((CkType == 3) && (autoread != 0))
	{
		//check status to see if currently in reader mode (ALL)
		//if read ltc and vitc, make sure both status bits are set
		//if the bits are not set, write the 'autoread' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x03))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x02)) //vitc
			ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x01)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(autoread)) && (i<5))
				i++;
		}
	}
	else if ((CkType == 2) || (CkType == 1))
		ok = TRUE;
	else ok = FALSE;
	return (ok);

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::AutoGen() //autogen
{
	
	if ((CkType == 3) && (autogen != 0))
	{
		//check status to see if currently in gen mode (ALL)
		//if gen ltc and vitc, make sure both status bits are set
		//if the bits are not set, write the 'autogen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x0C))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x08)) //vitc
			ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x04)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(autogen)) && (i<5))
				i++;
		}
	}
	else ok = FALSE;
	return (ok);

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::GenModeStat() //gen mode status check
{
	
	if (CkType == 3)
	{
		//check status to see if currently in gen mode 
		//if gen ltc or vitc, make sure both status bits are set

		if ((*(lpBrd + 0x3E8) & 0x0C) != 0x00)	//
			return TRUE;
		else return FALSE;
	}
	else return FALSE;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LJam()	//ltcjam
{

	if ((CkType == 3) && (ltcjam != 0))
	{

		i = 0;
		while (!(ok = SendCom(ltcjam)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
	

}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LReadMode() //ltcread
{
	if ((CkType == 3) && (ltcread != 0))
	{
		//check status to see if currently in reader mode (LTC)
		//if read ltc and vitc, make sure only LTC status bit is set
		//if the bit is not set, write the 'ltcread' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x01))	//vltc
			ok = TRUE;

		//else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x02)) //vitc
		//	ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x01)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(ltcread)) && (i<5))
				i++;
		}
	}
	else if ((CkType == 2) || (CkType == 1))
		ok = TRUE;
	else ok = FALSE;
	return (ok);
	

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::VReadMode() //vitcread
{
	if ((CkType == 3) && (vitcread != 0))
	{
		//check status to see if currently in reader mode (VITC)
		//if read ltc and vitc, make sure only vitc status bit is set
		//if the bit is not set, write the 'vitcread' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x02))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x02)) //vitc
			ok = TRUE;

		//else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x01)) //ltc
		//	ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(vitcread)) && (i<5))
				i++;
		}
	}
	else ok = FALSE;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LGenMode() //ltcgen
{
	if ((CkType == 3) && (ltcgen != 0))
	{
		//check status to see if currently in gen mode (LTC)
		//if read ltc and vitc, make sure only ltc status bit is set
		//if the bit is not set, write the 'ltcgen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x04))	//vltc
			ok = TRUE;

		//else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x04)) //vitc
		//	ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x04)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(ltcgen)) && (i<5))
				i++;
		}
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LRnGMode() //ltcrg
{
	if ((CkType == 3) && (ltcrg != 0))
	{
		//check status to see if currently in reader/gen mode (LTC)
		//if read and gen ltc, make sure only LTC status bits are set
		//if the bit is not set, write the 'ltcrg' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x05))	//vltc
			ok = TRUE;

		//else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x04)) //vitc
		//	ok = TRUE;	// ltcrg is 0 if VITC board

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x05)) //ltc
			ok = TRUE;
		
		else
		{
			if (Model == 0x98)
			{
				OneBR = *(lpBrd + 0x045);
				*(lpBrd + 0x045) = (OneBR | 0x01);	//only alter bit0
			}
			i = 0;
			while (!(ok = SendCom(ltcrg)) && (i<5))
				i++;
		}
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::VtoLMode() //vtol
{
	if ((CkType == 3) && (Model == 0x50) && (vtol != 0))
	{
		BYTE d;
	/*
	write 19h to offset 48h
	load generator default data
	send MODE command
	send RUN command
	*/
		if (!Idle())
			d =bDbug;
		Write1(0x19, 0x0048);	//force continuous jamming, Synced to video
		Write4(0x00000000, 0x0040);	//defualt time
		Write4(0x00000000, 0x0060);	//default user

		i = 0;
		while (!(ok = SendCom(vtol)) && (i<5))
			i++;
		i = 0;
		while (!(ok = SendCom(jamgenall)) && (i<5))
			i++;
		i = 0;
		while (!(ok = SendCom(genstart)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::VGenMode() //vitcgen
{
	if ((CkType == 3) && (vitcgen != 0))
	{
		//check status to see if currently in gen mode (VITC)
		//make sure only VITC status bit is set
		//if the bit is not set, write the 'vitcgen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && ((OneBR & 0x0F) == 0x08))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x08)) //vitc
			ok = TRUE;

		//else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x05)) //ltc
		//	ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(vitcgen)) && (i<5))
				i++;
		}
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LtoVMode() //ltov
{
	BYTE d;
	if ((CkType == 3) && (Model == 0x50) && (ltov != 0))
	{
	/*
	write 19h to offset 48h
	load generator default data
	send MODE command
	send RUN command
	*/
		if (!Idle())
			d =bDbug;
		Write1(0x19, 0x0048);	//force continuous jamming
		Write4(0x00000000, 0x0040);	//defualt time
		Write4(0x00000000, 0x0060);	//default user

		i = 0;
		while (!(ok = SendCom(ltov)) && (i<5))
			i++;
		if (!ok)
			d =bDbug;
		i = 0;
		while (!(ok = SendCom(jamgenall)) && (i<5))
			i++;
		if (!ok)
			d =bDbug;

		i = 0;
		while (!(ok = SendCom(genstart)) && (i<5))
			i++;
		if (!ok)
			d =bDbug;

	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// TODO: Add function to allow parameter to be passed
BOOL CAec::JamTU() //jamgenall
{
	if ((CkType == 3) && (jamgenall != 0))
	{
		i = 0;
		while (!(ok = SendCom(jamgenall)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// TODO: Add function to allow parameter to be passed
BOOL CAec::JamTB() //jamgentb
{
	if ((CkType == 3) && (jamgentb != 0))
	{
		i = 0;
		while (!(ok = SendCom(jamgentb)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// TODO: Add function to allow parameter to be passed
BOOL CAec::JamUB() //jamgenub
{
	if ((CkType == 3) && (jamgenub != 0))
	{
		i = 0;
		while (!(ok = SendCom(jamgenub)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Start() //genstart
{
	if ((CkType == 3) && (genstart != 0))
	{
		//check status to see if currently in gen mode (any)
		//if gen ltc or vitc bits are set, do not change,
		//if a gen bit is not set, write the 'autogen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && (((OneBR & 0x0C) == 0x08) || 
		  ((OneBR & 0x0C) == 0x04) || ((OneBR & 0x0C) == 0x0C)))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x08)) //vitc
			ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x04)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(autogen)) && (i<5))
				i++;
		}

		i = 0;
		if (ok)
		{
			while (!(ok = SendCom(genstart)) && (i<5))	// start
				i++;
		}
	}
	else ok = FALSE;
	return (ok);	

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Start(DWORD tb) //genstart
{
	if ((CkType == 3) && (genstart != 0))
	{
		//check status to see if currently in gen mode (any)
		//if gen ltc or vitc bits are set, do not change,
		//if a gen bit is not set, write the 'autogen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && (((OneBR & 0x0C) == 0x08) || 
		  ((OneBR & 0x0C) == 0x04) || ((OneBR & 0x0C) == 0x0C)))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x08)) //vitc
			ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x04)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(autogen)) && (i<5))
				i++;
		}

		*((LPDWORD)(lpBrd + 0x040)) = tb;	// write time data to gen input
		i = 0;
		while (!(ok = SendCom(jamgentb)) && (i<5))	// jam time
			i++;		
		
		if (ok)
			{
				i = 0;
				while (!(ok = SendCom(genstart)) && (i<5))	// start
					i++;
			}
	}
	else ok = FALSE;
	return (ok);	
	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Start(DWORD tb, DWORD ub) //genstart
{
	if ((CkType == 3) && (genstart != 0))
	{
		//check status to see if currently in gen mode (any)
		//if gen ltc or vitc bits are set, do not change,
		//if a gen bit is not set, write the 'autogen' byte to 3FF

		OneBR = *(lpBrd + 0x3E8);
		if ((Model == 0x50) && (((OneBR & 0x0F) == 0x08) || 
		  ((OneBR & 0x0F) == 0x04) || ((OneBR & 0x0F) == 0x0C)))	//vltc
			ok = TRUE;

		else if ((Model == 0x93) && ((OneBR & 0x0F) == 0x08)) //vitc
			ok = TRUE;

		else if ((Model == 0x98) && ((OneBR & 0x0F) == 0x04)) //ltc
			ok = TRUE;
		
		else
		{
			i = 0;
			while (!(ok = SendCom(autogen)) && (i<5))
				i++;
		}

		*((LPDWORD)(lpBrd + 0x040)) = tb;	// write time data to gen input
		*((LPDWORD)(lpBrd + 0x060)) = ub;	// write user data to gen input
		i = 0;
		while (!(ok = SendCom(jamgenall)) && (i<5))	// jam time/user
			i++;		
		i = 0;
		if (ok)
			{
				while (!(ok = SendCom(genstart)) && (i<5))	// start
					i++;
			}
	}
	else ok = FALSE;
	return (ok);		

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::StopGen() //genstop
{
	if ((CkType == 3) && (genstop != 0))
	{
		i = 0;
		while (!(ok = SendCom(genstop)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);		

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::ResetBrd() //reset
{
	if (CkType == 3)
	{
		i = 0;
		while (!(ok = SendCom(reset)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);		
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::EnableNext() //intack
{
	if (CkType == 3)
	{
		i = 0;
		while (!(ok = SendCom(intack)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Smpte() //smpte
{
	if (CkType == 3)
	{
		i = 0;
		while (!(ok = SendCom(smpte)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Ebu() //ebu
{
	if (CkType == 3)
	{
		i = 0;
		while (!(ok = SendCom(ebu)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Idle() //idle
{
	if (CkType == 3)
	{
		i = 0;
		while (!(ok = SendCom(idle)) && (i<5))
			i++;
	}
	else ok = FALSE;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::InitSonySerial()	//inituart 38400/8/o/1/RS422 TX enabled
{
	if (!Serial)
		return FALSE;
	*(lpBrd + 0x0103) = 0xA0;	//  8/ODD
	*(lpBrd + 0x0102) = 8;		//  38400 baud
	*(lpBrd + 0x0100) = 4;		//  RS422
	*(lpBrd + 0x010D) = 0x01;	//  Set bit0 to leave TX enabled
	i = 0;
	while (!(ok = SendCom(inituart)) && (i<5))
		i++;
	SonySetup = ok;	//will flag commands to call this function if not set.
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::InitSerial()	//inituart
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(inituart)) && (i<5))
		i++;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Baud = 38400, 19200, 9600, 4800, 2400, 1200, 600, 300 or 150
// Bits = 7 or 8
// P = o, O, e, E, n or N	(parity is ODD, EVEN or NONE)
BOOL CAec::InitSerial(int Baud, int Bits, char P)	//inituart
{
	if (!Serial)
		return FALSE;

	if ((P == 'e') || (P == 'E'))
		OneBW = 0x00;
	else if ((P == 'n') || (P == 'N'))
		OneBW = 0x40;
	else //((P == 'o') || (P == 'O'))
		OneBW = 0x80;

	if ((Bits == 7) && ((P == 'n') || (P == 'N')))
		return FALSE;	//cannot have 7bits no Parity
	else if (Bits == 7)
		OneBW = (OneBW & 0xDF);	//clears bit 5
	else OneBW = (OneBW | 0x20);	//sets bit 5
	*(lpBrd + 0x0103) = OneBW;

	if (Baud >= 38400)
		OneBW = 8;
	else if (Baud >= 19200)
		OneBW = 7;
	else if (Baud >= 9600)
		OneBW = 6;
	else if (Baud >= 4800)
		OneBW = 5;
	else if (Baud >= 2400)
		OneBW = 4;
	else if (Baud >= 1200)
		OneBW = 3;
	else if (Baud >= 600)
		OneBW = 2;
	else if (Baud >= 300)
		OneBW = 1;
	else if (Baud >= 300)
		OneBW = 0;
	else OneBW = 8;	//38400 baud default
	*(lpBrd + 0x0102) = OneBW;

	i = 0;
	while (!(ok = SendCom(inituart)) && (i<5))
		i++;
	return (ok);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Baud = 38400, 19200, 9600, 4800, 2400, 1200, 600, 300 or 150
// Bits = 7 or 8
// P = o, O, e, E, n or N	(parity is ODD, EVEN or NONE)
// M = 0, 2, 4 or 5 (ESbus, RS232, RS422, RS485)
BOOL CAec::InitSerial(int Baud, int Bits, char P, int M)	//inituart
{
	if (!Serial)
		return FALSE;
	if ((P == 'e') || (P == 'E'))
		OneBW = 0x00;
	else if ((P == 'n') || (P == 'N'))
		OneBW = 0x40;
	else //((P == 'o') || (P == 'O'))
		OneBW = 0x80;

	if ((Bits == 7) && ((P == 'n') || (P == 'N')))
		return FALSE;	//cannot have 7bits no Parity
	else if (Bits == 7)
		OneBW = (OneBW & 0xDF);	//clears bit 5
	else OneBW = (OneBW | 0x20);	//sets bit 5
	*(lpBrd + 0x0103) = OneBW;

	if (Baud >= 38400)
		OneBW = 8;
	else if (Baud >= 19200)
		OneBW = 7;
	else if (Baud >= 9600)
		OneBW = 6;
	else if (Baud >= 4800)
		OneBW = 5;
	else if (Baud >= 2400)
		OneBW = 4;
	else if (Baud >= 1200)
		OneBW = 3;
	else if (Baud >= 600)
		OneBW = 2;
	else if (Baud >= 300)
		OneBW = 1;
	else if (Baud >= 300)
		OneBW = 0;
	else OneBW = 8;	//38400 baud default
	*(lpBrd + 0x0102) = OneBW;

	if ((M == 0) || (M == 2) || (M == 4) || (M == 5))
		*(lpBrd + 0x0100) = (BYTE)M;


	i = 0;
	while (!(ok = SendCom(inituart)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// !*!*!*!*!*!  WRITE FUNCTION FOR ENHANCED SERIAL MODE TOO !*!*!*!*!*!
BOOL CAec::InitRx()	//initrx
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(initrx)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Passes back the TWO bytes as a WORD, low byte is status
//							high byte is the byte that was received.
WORD CAec::RxByte()	//readrx
{
	if (!Serial)
		return 0x00FF;
	i = 0;
	while (!(ok = SendCom(readrx)) && (i<5))
		i++;
	RX = (WORD)((*(lpBrd + 0x0109) * 0x100) + *(lpBrd + 0x0108));
	return RX;	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BYTE CAec::TxStatus()	//txstat
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstat)) && (i<5))
		i++;
	return *(lpBrd + 0x010C);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxEn()	//tx422en
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(tx422en)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxDis()	//tx422dis
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(tx422dis)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxBreak()	//txbreak
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txbreak)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxByte(BYTE txOne)	//txbyte
{
	if (!Serial)
		return FALSE;
	*(lpBrd + 0x010E) = txOne;	// store the byte to be transmitted
	i = 0;
	while (!(ok = SendCom(txbyte)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS1()	//txstr1
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr1)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS2()	//txstr2
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr2)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS3()	//txstr3
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr3)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS4()	//txstr4
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr4)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS5()	//txstr5
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr5)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS6()	//txstr6
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr6)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS7()	//txstr7
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr7)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS8()	//txstr8
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr8)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS9()	//txstr9
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstr9)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS10()	//txstrA
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrA)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS11()	//txstrB
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrB)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS12()	//txstrC
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrC)) && (i<5))
		i++;
	return (ok);		
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS13()	//txstrD
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrD)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS14()	//txstrE
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrE)) && (i<5))
		i++;
	return (ok);		
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxS15()	//txstrF
{
	if (!Serial)
		return FALSE;
	i = 0;
	while (!(ok = SendCom(txstrF)) && (i<5))
		i++;
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// The byte that is returned will be 0x00 if no errors occured.
// The return value will be NON-ZERO IF ERROR occurs with the error
//  being flaged in a bitwise encoding.
//	 bit7 = data vaild, (will only be set if bit 0, 1, or 2 are also set
//	 bit6 = for TIMEOUT			(0x40)
//	 bit5 = NAK					(0x20)
//	 bit2 = buffer overflow		(0x8x)
//	 bit1 = break character rxed	"
//	 bit0 = parity error			"
// The string 'S' will be loaded with the received data, without null termination
BYTE CAec::RxSony(char* S)
{
	int ii, i, iCnt;
	//LONG toc;
	CTimer to;

	if (!Serial)
		return FALSE;

	iCnt = 1;
	to.Zero();		//zero timer, Start of timeout routine
	for (i=0; (i<iCnt); i++)
	{
		//ii = 0;
		while (!(ok = SendCom(readrx)) && ( to.msT() <= 10))  //(ii<5))
			ii++;
		if (ok)
		{
			if ( *(lpBrd + 0x0108) == 0x80 )	// command and RX are error free
			{
				to.Zero();		//zero timer, re-start timeout routine
				*(S+i) = (char) *(lpBrd + 0x0109);	//save rx byte
				if (i == 0)
					iCnt = (int)*S + 3;	// Save the number of bytes to expect						
			}
			else return *(lpBrd + 0x0108);	// Command ok, but RX error
											// status byte
		}	
		else return 0x40;	// bit6 for TIMEOUT
	}
	if ((*(S+0) == 0x11) && (*(S+1) == 0x12))
	{
		//rx_nak++;		//count of NAK errors
		return 0x20;	// bit5 for NAK
	}
	else return 0x00;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// This function is passed a pointer to a string received from "sony" device.
// This code checks to see if this string is ok and if it is a NAK.
// The byte that is returned will be 0x00 if no errors occured.
// The return value will be NON-ZERO IF ERROR occurs with the error
//  codes as follows:
//		OK = TRUE;
//		nak or other = FALSE;
BOOL CAec::CheckSonyRX(LPSTR lpStr)
{
	int i, iCnt;
	BYTE cmd1, cmd2, cks, cksByte;

	cks = 0;	//start with zero
	cmd1 = lpStr[0];
	cmd2 = lpStr[1];
	iCnt = (int)((lpStr[0] & 0x0F) + 2);	//do not include checksum
	for (i=0; (i<iCnt); i++)
	{
		cks += lpStr[i];
	}
	//if (cks != lpStr[iCnt + 1])
	cksByte = lpStr[iCnt];
	if ( cksByte != cks)
		return FALSE;	//undefined, bad
	else 
	{
		if ( (cmd1 == 0x11) && (cmd2 == 0x12) )
			return FALSE;
		else return TRUE;
	}
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// If some error occurs in the RX of data, see if it is a NAK response, and if
// it is, get the nak data byte.
// This function is passed a pointer to a string received from "sony" device.
// This code checks to see if this string is ok and if it is a NAK.
// The byte that is returned will be 0x00 if not a NAK.
// The return value will be NON-ZERO IF ERROR occurs with the error
//  codes as follows:
//		unknown error = 00h;
//		NAK = Nak data byte;
// This function should be called only if a FALSE is received from "CheckSonyRX" above
BYTE CAec::SonyNAK(LPSTR lpStr)
{
	int i, iCnt;
	BYTE cmd1, cmd2, cks;

	cks = 0;	//start with zero
	cmd1 = lpStr[0];
	cmd2 = lpStr[1];
	iCnt = (int)((lpStr[0] & 0x0F) + 2);	//do not include checksum
	for (i=0; (i<iCnt); i++)
	{
		cks += lpStr[i];
	}
	if (cks != lpStr[iCnt + 1])
		return 0x00;	//undefined, bad
	else 
	{
		if ( (cmd1 == 0x11) && (cmd2 == 0x12) )
			return (lpStr[2]);
		else return 0x00;	//string ok, but not a NAK
	}
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Will Load and Transmit Sony command String
BOOL CAec::TxSony(int j, char* B)	//txstr1-F  j=1-F
{
	BYTE cks;
	int i;

	if (!Serial)
		return FALSE;
	if ((j < 0x01) || (j > 0x0F))
		return FALSE;
	OneDummy = (BYTE)((B[0] & 0x0F) + 3);	// calculate first byte (length)
	// write the string length to first byte position
	OffsetW = (0x0100 + (j * 0x10));
	*(lpBrd + OffsetW) = OneDummy;
	cks = 0;	// initialize checksum
	for (i = 0; i < (OneDummy - 1); i++)
	{
		OffsetW = (0x0101 + (j * 0x10) + i);
		*(lpBrd + OffsetW) = (BYTE)B[i];
		cks = cks + (BYTE)B[i];
	}
	// Write the checksum byte to the string
	OffsetW = (0x0100 + (j * 0x10) + OneDummy);
	*(lpBrd + OffsetW) = cks;
	i = 0;
	OneBW = (0xA0 + (BYTE)j);
	while (!(ok = SendCom(OneBW)) && (i<5))
		i++;
	return (ok);			
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TxSony(int j)	//txstr1-F  i=1-F
{
	if (!Serial)
		return FALSE;
	if ((j < 0x01) || (j > 0x0F))
		return FALSE;
	i = 0;
	OneBW = (0xA0 + (BYTE)j);
	while (!(ok = SendCom(OneBW)) && (i<5))
		i++;
	return (ok);		
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Transfer byte data from string B into TX buffer i at offset address 
//  0xAi of Board at Address.
// determines the length byte [0] and the checksum on the fly and
// sets those values in the tx string given the cmd1, cmd2 and data bytes.
BOOL CAec::LoadSonyStr(int j, char *B)	//
{
	BYTE cks;
	int i;
	if (!Serial)
		return FALSE;
	if ((j < 0x01) || (j > 0x0F))
		return FALSE;
	OneDummy = (BYTE)((B[0] & 0x0F) + 3);	// calculate first byte (length)
	// write the string length to first byte position
	OffsetW = (0x0100 + (j * 0x10));
	*(lpBrd + OffsetW) = OneDummy;
	cks = 0;	// initialize checksum
	for (i = 0; i < (OneDummy - 1); i++)
	{
		OffsetW = (0x0101 + (j * 0x10) + i);
		*(lpBrd + OffsetW) = (BYTE)B[i];
		cks = cks + (BYTE)B[i];
	}
	// Write the checksum byte to the string
	OffsetW = (0x0100 + (j * 0x10) + OneDummy);
	*(lpBrd + OffsetW) = cks;
	return TRUE;
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BYTE CAec::ModeStatus()	//reads from offset 3E8h
{
	if (CkType == 3)
		OneBR = *(lpBrd + 0x3E8);
	else if ((CkType == 1) || (CkType == 2))
		OneBR = 0x01;	//ltc reader if IOR or MMR
	else OneBR = 0x00;
	return OneBR;		

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
int CAec::Refresh()
{
	Find();
	return (Select(selection));

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
//SendCom...
//	This function takes a command byte as its parameter and will return 
//	a TRUE if the command that is sent is read (ID) at offset 3FE before a
//	a timeout has occured. If the ID does not ever equal the command
//	before a timeout occurs, a FALSE is returned. 
//	Special cases: If the command is 0 or 2, then the value is written
//	and a TRUE is returned. This is because these command values do not
//	return an ACK.  If a TRUE is returned, it can be assumed that the
//	ComAck value is the same as the command, if FALSE is returned, then
//	read the ComAck value upon return and see what happened. If ComAck is
//	a 0xFx value, then there was a NAK.
// ----------------------------------------------------------------------
BOOL CAec::SendCom(BYTE combyte)
{
	LONG toc;
	CTimer tmSC;
	if (!pointer)
		return FALSE;
	if (ISRRun)
		goto isr_ack;	//JUMP 
	  
	// SET INTERRUPT is COMING FLAG if Applicable
	//IntPending = FALSE;
	//if ((combyte != 0x02) || (combyte != 0x00))	  //if NOT (ACK or Reset) ==> TRUE
	//	IntPending = TRUE;
		
	//wait for 'FF'
	toc = 0x00000000;
	while (( *(lpBrd + 0x3FF) != 0xFF) && (toc < 0x3FFFF))
		++toc;
	if (toc >= 0x3FFFF)
	{
		MessageBeep(0xFFFFFFFF);	//FOR DEBUG
		return FALSE;
	}

	ComAck = 0x00;					// DEFAULT Ack value (FALSE)
	OneDummy = *(lpBrd + 0x3FF);	// read again, for timing
	//*(lpBrd + 0x3FD) = 0x00;		// clear intPendingFlag
	*(lpBrd + 0x3FF) = combyte;		// send command
	  
	// these values do not get ACKed, so do not wait around for ACK.
	if ((combyte == 0x00) || (combyte == 0x02))	
		return TRUE;			

	tmSC.Zero();		//zero timer, Start of timeout routine
			
sc_lp: 
		// wait for interrupt pending flag to be set (get the ACK)
		// or wait till timeout (waiting too long) 
		// (Default timeout value is 50ms)
	do {
		OneVol = *(lpBrd + 0x3FD);
	}
	while ((OneVol == 0x00) && ( (tmrVol = tmSC.msT()) < ComTimeout));


	if ( OneVol == 0x00 )	//this is a timeout error
	{
		bDbug = *(lpBrd + 0x3FE);	//temp debugging!
		//OneDum2 = *(lpBrd + 0x3F0);	//temp debugging!
		MessageBeep(0xFFFFFFFF);
		return FALSE;			
	}
	
	else 
	{
		ComAck = *(lpBrd + 0x3FE);	// get the Interupt Code
		// Reset the command input and clear the interrupt pending
		// byte

			//Wait for 'FF' or timeout
		//toc = 0x00000000;
		while (( *(lpBrd + 0x3FF) != 0xFF) && ( (tmrVol = tmSC.msT()) < ComTimeout))
			++toc; //dummy operation

		*(lpBrd + 0x3FD) = 0x00;	//clear int flag
		*(lpBrd + 0x3FF) = 0x00;	//enable next interrupt
		if (ComAck != combyte)		//if current int is not equal to command	
			goto sc_lp;				// then loop until match or timeout (MAX)
		return(TRUE);				// this is what we wanted
	}

isr_ack:
	if (combyte == 0x00)
		return TRUE;		// Writing 0x00 to 3FF under ISR conditions
							// does nothing, The ISR must always leave
							// the board in a ready state anyway
							// The ISR_n function of the .DLL file does this!
 //NOTE----
  // The .dll function that handles interrupts does not currently handle
  // this case well. The interrupt can be cued and return to the Main thread
  // prior to 3FF being FF (can return with it just having been written with 00)
  // There is currently no way that I know to flag this. 

	  // SET INTERRUPT is COMING FLAG if Applicable
	IntPending = FALSE;
	if (combyte != 0x02)	  //if NOT (Reset) ==> TRUE
		IntPending = TRUE;

		//wait for 'FF'
	toc = 0x00000000;
	while (( *(lpBrd + 0x3FF) != 0xFF) && (toc < 0x3FFFF))
		++toc;
	// ************ Critical from here for the next several lines if ISR hits!
	// If ISR thread is called here, it may be possible to try and write the
	// command byte ('combyte') to 3FF without 3FF = 0xFF.
	// If ISR thread is in use, make sure that the ISR sets the  'ComAck' value
	// when a command response ISR is handled. When ISRRUN is true, the line of
	// code that calls this function needs to wait for ComAck to be set or try
	// to send the command again if a timeout period has elapsed
	// The return value will be false if the ComAck value does not equal the 
	// command before the timeout period has elapsed. Try to send command again
	if (toc >= 0x3FFFF)
	{
		IntPending = FALSE;
		return FALSE;
	}
	ComAck = 0x00;					// default Ack value
	OneDummy = *(lpBrd + 0x3FF);	// read again, for timing
	//*(lpBrd + 0x3FD) = 0x00;		// clear intPendingFlag
	*(lpBrd + 0x3FF) = combyte;		// send command
	// ************ Critical above here

	tmSC.Zero();
	while ((ComAck != combyte) && ((tmrVol = tmSC.msT()) < IntTimeout ))
		++toc;	//dummy operation

	if (ComAck == combyte)	// This is what we wanted
	{
		IntPending = FALSE;
		return TRUE;
	}
	else 
	{
		IntPending = FALSE;
		return FALSE;	//this indicates a timeout, The ISR did not set
						//  this value before the timout period!!
	}
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
char* CAec::GetStrId(char* str)  // pass back the string with board ID
{
	return strcpy( str, strId );
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
char* CAec::Get3ExIDBytes(char* str)
{
	str[0] = (char)Id0;
	str[1] = (char)Id1;
	str[2] = (char)Id2;
	str[3] = (char)SLtr;
	str[4] = (char)SNum;
	str[5] = (char)VerUp;
	str[6] = (char)VerLow;
	str[7] = ' ';
	str[8] = ' ';
	str[9] = ' ';
	str[10] = ' ';
	str[11] = ' ';
	str[12] = ' ';
	str[13] = ' ';
	str[14] = ' ';
	str[15] = (char)Model;	//
	str[16] = 0;	//null
	return str;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::SerialOp()
{
	return (Serial);
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Play()			//play
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	//char SonyStr[] = {0x20, 0x01};
	if (ok)
	{
		char SonyStr[] = {0x20, 0x01};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Stop()			//stop
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x04};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Pause()			//pause
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x00};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Record()		//record
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x02};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::FastForward()	//fast_forward
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x10};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Rewind()		//rewind_tape
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x20};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Eject()			//eject
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x20, 0x0F};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Tmr1_reset()	//timer1 reset
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x40, 0x08};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Tmr1()			//ctl/timer1 time sense
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x61, 0x0C, 0x04};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::LTCc()			//ltc time sense
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x61, 0x0C, 0x01};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::VITCc()			//vitc time sense
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x61, 0x0C, 0x02};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::Status()		//vtr status sense
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x61, 0x20, 0x09};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
//
BOOL CAec::Status(BYTE stat)	//vtr status sense
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x61, 0x20, stat};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// TODO: create additional function to accept ascii and int data
BOOL CAec::Cue(BYTE hh, BYTE mm, BYTE ss, BYTE ff)	//cue/goto HH:MM:SS:FF
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x24, 0x31, ff, ss, mm , hh};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::ShtlF(BYTE speed)	//shuttle forward at 'speed'
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x21, 0x13, speed};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::ShtlR(BYTE speed)	//shuttle reverse at 'speed'
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x21, 0x23, speed};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::UseTC()			//set to use Timecode (not CTL) mode
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x41, 0x36, 0x00};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::UseCtl()		//set to use Timer1/CTL (not timecode) mode
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x41, 0x36, 0x01};
		ok = TxSony(15, SonyStr);
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
BOOL CAec::TMS()			//request time mode sense (currently in use)
{
	if (!SonySetup)
		ok = InitSonySerial();	// will also check for 'Serial'
	if (ok)
	{
		char SonyStr[] = {0x60, 0x36};
		ok = TxSony(15, SonyStr);	
		SonySetup = ok;	//causes re-init of serial next time if error
	}
	return (ok);	
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// transfer string 'chS' of 'Len' length to TX buffer 'j'
BOOL CAec::LoadTxString(int j, int Len, char* chS)
{
	int i;
	if (!Serial)
		return FALSE;
	if ((j < 0x01) || (j > 0x0F))	//must select string buffer 1-F
		return FALSE;
	if ((Len > 0x0F))	//must have length less than 255 bytes
		return FALSE;

	// write the string length to first byte position
	OffsetW = (0x0100 + (j * 0x10));
	*(lpBrd + OffsetW) = (BYTE)Len;
	OffsetW++;	//add one to offset
	for (i = 0; i <= Len; i++)
	{
		*(lpBrd + OffsetW + i) = (BYTE)chS[i];	//write string to tx buffer
	}
	return TRUE;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// Bit field that passes Lrdr/gen & Vrdr/gen capability bits
// similar to byte at offset 3E8 (mode status) low nibble
int CAec::TCmodes()					
{
	int k = 0;
	if	(LRdr)				//True if board reads LTC
		k = (k | 0x0001);	// set bit0
	if	(LGen)				//True if board gens LTC
		k = (k | 0x0002);	// set bit1
	if	(VRdr)				//True if board reads VITC
		k = (k | 0x0004);	// set bit2
	if	(VGen)				//True if board gens VITC
		k = (k | 0x0008);	// set bit3
	return k;
}
/*
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
CAec::
{
	

}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
*/


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
CTimer::CTimer()
{
//	ok = FALSE;
	ok = QueryPerformanceFrequency(&lpFq);
			// counts/sec		// address of current frequency
			// lpFq.LowPart can handle up to 4 GHz in the 32 bits.
	ok = QueryPerformanceCounter(&lpCnt1);
	lpCnt1.LowPart = (lpCnt1.LowPart & 0x7FFFFFFF); //Mask high bit for rollover
}
// ----------------------------------------------
BOOL CTimer::Zero()
{
	ok = QueryPerformanceCounter(&lpCnt1);
	lpCnt1.LowPart = (lpCnt1.LowPart & 0x7FFFFFFF); //Mask high bit for rollover
	return (ok);
}
// ----------------------------------------------
int CTimer::msT()
{
	ok = QueryPerformanceCounter(&lpCnt2);
	lpCnt2.LowPart = (lpCnt2.LowPart & 0x7FFFFFFF); //Mask high bit for rollover
	if ( lpCnt2.LowPart < lpCnt1.LowPart )			// handle the rollover case			
		lpCnt2.LowPart = (lpCnt2.LowPart | 0x80000000);					
	return (int)((lpCnt2.LowPart - lpCnt1.LowPart) / (lpFq.LowPart/1000));
}
// ----------------------------------------------
DWORD CTimer::SysTime(int frames)
{
	CStrHex bcd;
	SYSTEMTIME	pcTime;	//time/date structure used to read from and pass
	BYTE	bHr, bMn, bSec, bFrm;

	GetSystemTime(&pcTime);
	//GetLocalTime(&pcTime);	// get the current local (pc) time

	bHr = bcd.WordBCD(pcTime.wHour);
	bMn = bcd.WordBCD(pcTime.wMinute);
	bSec = bcd.WordBCD(pcTime.wMilliseconds);
	bFrm = bcd.WordBCD(pcTime.wMilliseconds / (1000/(WORD)frames));

	return (DWORD)((bHr * 0x1000000) + (bMn * 0x10000) + (bSec * 0x100) + bFrm );
}
// ----------------------------------------------
DWORD CTimer::LocalTime(int frames)
{
	CStrHex bcd;
	SYSTEMTIME	pcTime;	//time/date structure used to read from and pass
	BYTE	bHr, bMn, bSec, bFrm;

	//GetSystemTime(&pcTime);
	GetLocalTime(&pcTime);	// get the current local (pc) time

	bHr = bcd.WordBCD(pcTime.wHour);
	bMn = bcd.WordBCD(pcTime.wMinute);
	bSec = bcd.WordBCD(pcTime.wMilliseconds);
	bFrm = bcd.WordBCD(pcTime.wMilliseconds / (1000/(WORD)frames));

	return (DWORD)((bHr * 0x1000000) + (bMn * 0x10000) + (bSec * 0x100) + bFrm );
}
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	

//	String / Character translation class functions
// -----------------------------------------------
CStrHex::CStrHex()
{
	i = 0;
}
// -----------------------------------------------
//receive each string and check each character for a '0'-'F' value
// if character is "out of range" low set to '0', else 'F'
DWORD CStrHex::ToHex(char *Ath)
{
	i = strlen(Ath);
	if (i == 1)	//convert to hex BYTE
	{
		by[3] = 0;
		by[2] = 0;
		by[1] = 0;
		by[0] = CHex(*Ath);
		//return (VOID*)(&B);					
	}
	else if (i == 2)	//convert to hex BYTE
	{
		by[3] = 0;
		by[2] = 0;
		by[1] = 0;
		by[0] = ((CHex(*(Ath+0)) * 0x10) + (CHex(*(Ath+1))));
		//return (VOID*)(&B);					
	}
	else if (i == 3)	//convert to hex BYTE
	{
		by[3] = 0;
		by[2] = 0;
		by[1] = CHex(*(Ath + 0));
		by[0] = ((CHex(*(Ath+1)) * 0x10) + (CHex(*(Ath+2))));
		//return (VOID*)(&B);					
	}
	else if (i == 4)	//convert to hex BYTE
	{
		by[3] = 0;
		by[2] = 0;
		by[1] = ((CHex(*(Ath+0)) * 0x10) + (CHex(*(Ath+1))));
		by[0] = ((CHex(*(Ath+2)) * 0x10) + (CHex(*(Ath+3))));
		//return (VOID*)(&B);					
	}
	else if (i == 5) //assume that there are 8 'hex' chars only
	{
		by[3] = 0;
		by[2] = CHex(*Ath);
		by[1] = ((CHex(*(Ath+1)) * 0x10) + (CHex(*(Ath+2))));
		by[0] = ((CHex(*(Ath+3)) * 0x10) + (CHex(*(Ath+4))));
		//return (VOID*)(&B);					
	}
	else if (i == 6) //assume that there are 8 'hex' chars only
	{
		by[3] = 0;
		by[2] = ((CHex(*(Ath+0)) * 0x10) + (CHex(*(Ath+1))));
		by[1] = ((CHex(*(Ath+2)) * 0x10) + (CHex(*(Ath+3))));
		by[0] = ((CHex(*(Ath+4)) * 0x10) + (CHex(*(Ath+5))));
		//return (VOID*)(&B);					
	}
	else if (i == 7) //assume that there are 8 'hex' chars only
	{
		by[3] = CHex(*(Ath+0));
		by[2] = ((CHex(*(Ath+1)) * 0x10) + (CHex(*(Ath+2))));
		by[1] = ((CHex(*(Ath+3)) * 0x10) + (CHex(*(Ath+4))));
		by[0] = ((CHex(*(Ath+5)) * 0x10) + (CHex(*(Ath+6))));
		//return (VOID*)(&B);					
	}

	else if ((i >= 8) && (i < 11)) //assume that there are 8 'hex' chars only
	{
		by[3] = ((CHex(*(Ath+0)) * 0x10) + (CHex(*(Ath+1))));
		by[2] = ((CHex(*(Ath+2)) * 0x10) + (CHex(*(Ath+3))));
		by[1] = ((CHex(*(Ath+4)) * 0x10) + (CHex(*(Ath+5))));
		by[0] = ((CHex(*(Ath+6)) * 0x10) + (CHex(*(Ath+7))));
		//return (VOID*)(&B);					
	}
	else //if (i >=11 )	//work with first 8 'hex' characters 
	{				// ie: hh:mm:ss:ff ==> (DWORD)0xhhmmssff
		by[3] = ((CHex(*(Ath+0)) * 0x10) + (CHex(*(Ath+1))));
		by[2] = ((CHex(*(Ath+3)) * 0x10) + (CHex(*(Ath+4))));
		by[1] = ((CHex(*(Ath+6)) * 0x10) + (CHex(*(Ath+7))));
		by[0] = ((CHex(*(Ath+9)) * 0x10) + (CHex(*(Ath+10))));
		//return (VOID*)(&B);			
	}
	return (DWORD)((by[3] * 0x1000000) + (by[2] * 0x10000) + (by[1] * 0x100) + by[0]);
}
// -----------------------------------------------
BYTE CStrHex::CHex(char c)
{
	//if (c > 'F')
	c = toupper(c);
	if (c > 'F')
		return 0x0F;
	else if (c >= 'A')
		return (BYTE)((c - 0x41) + 10);	// 'A' = 0x41 ==> 0x0A = 10
	else if ((c >= '0') && (c <= '9'))
		return (BYTE)(c - 0x30);
	else return 0x00;
}
// -----------------------------------------------
DWORD CStrHex::IntBCD(int i)
{
	iw = (DWORD)i;
	for (j = 0; (((iw / 10) != 0) && (j < 8)); j++)
	{
		by[j] = (BYTE)(iw % 10);// save each decimal value of int in separate byte
		iw /= 10;				// drop 1's digit (Assignment Operator)
	}
	for (k = j; k != 0; k--);
	{
		dw = (dw * 0x10) + (DWORD)by[k - 1];	// mix bytes back into packed HEX DWORD

	}
	return dw;
}
// -----------------------------------------------
int CStrHex::TotalFrames(DWORD wDat, int frm)
{
	int i;
	for (i=0; i<8; i++)
	{
		by[i] = (BYTE)(wDat % 0x10);
		wDat /= 0x10;
	}
					//ff0 + ff10*10   + (ss1 + ss10*10)*frames
	return ( (int)( by[0] + (by[1] * 10) + ((by[2] + (by[3] * 10)) * frm)
		+ ((by[4] + (by[5]*10)) * frm * 60)	//mm * 60sec * frames/sec
		+ ((by[6] + (by[7]*10)) * frm * 3600)) ); //hh * 3600sec * frames/sec
}
// -----------------------------------------------
BYTE CStrHex::IntBcdByte(int iDat)
{
	return (((iDat / 10) * 16 ) + (iDat % 10));
}

// -----------------------------------------------
BYTE CStrHex::WordBCD(WORD wDat)
{
	return (((wDat / 10) * 16 ) + (wDat % 10));
}
// -----------------------------------------------
BYTE CStrHex::PackHex(char high, char low)
{
	return ((CHex(high) * 0x10) + CHex(low));
}
// -----------------------------------------------
// subtract the second packed BCD from the first. Return +- value as int
int CStrHex::FramesDiff(DWORD dw1, DWORD dw2, int frames)
{
	//int i1, i2; //i1 = TotalFrames(dw1, frames); //i2 = TotalFrames(dw2, frames);
	return 	( TotalFrames(dw1, frames) - TotalFrames(dw2, frames));
}
// -----------------------------------------------
char* CStrHex::BStr(BYTE bt, char *chs)
{
	wsprintf (chs,"%2.2X",bt);
	return chs;
}
// -----------------------------------------------
char* CStrHex::WStr(WORD wd, char *chs)
{
	wsprintf (chs,"%4.4X",wd);
	return chs;
}
// -----------------------------------------------
char* CStrHex::DWStr(DWORD dw, char *chs)
{
	wsprintf (chs,"%8.8X",dw);
	return chs;
}
// -----------------------------------------------
char* CStrHex::DWStrTm(DWORD dw, char *chs)
{
	wsprintf (chs,"%8.8X",dw);
	chs[11] = 0;	//EOL
	chs[10] = chs[7];
	chs[9] = chs[6];
	chs[8] = ':';
	chs[7] = chs[5];
	chs[6] = chs[4];
	chs[5] = ':';
	chs[4] = chs[3];
	chs[3] = chs[2];
	chs[2] = ':';

	return chs;
}

// -----------------------------------------------
char CStrHex::NibChar(BYTE bt)
{
	char c;
	switch (bt){
	case 0x00:
		c = '0';
		break;
	case 0x01:
		c = '1';
		break;
	case 0x02:
		c = '2';
		break;
	case 0x03:
		c = '3';
		break;
	case 0x04:
		c = '4';
		break;
	case 0x05:
		c = '5';
		break;
	case 0x06:
		c = '6';
		break;
	case 0x07:
		c = '7';
		break;
	case 0x08:
		c = '8';
		break;
	case 0x09:
		c = '9';
		break;
	case 0x0A:
		c = 'A';
		break;
	case 0x0B:
		c = 'B';
		break;
	case 0x0C:
		c = 'C';
		break;
	case 0x0D:
		c = 'D';
		break;
	case 0x0E:
		c = 'E';
		break;
	default:
		c = 'F';
		break;
	}
	return c;	
}

/*
// -----------------------------------------------
CStrHex::
{

}
// -----------------------------------------------
*/
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	

  